import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:math';
import 'package:airpay_emar_package/model/MerchantDetailResponse.dart';
import 'package:airpay_emar_package/model/QrDecryptResponse.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';
// Import for Android features.
import 'package:webview_flutter_android/webview_flutter_android.dart';
// Import for iOS features.
import 'package:airpay_emar_package/model/Transaction.dart';
import 'package:airpay_emar_package/model/UserRequest.dart';
import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:crypto/crypto.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:intl/intl.dart';
import 'package:xml2json/xml2json.dart';
import 'package:http/http.dart' as http;
import 'package:encrypt/encrypt.dart' as encrypt;
import 'package:crc32_checksum/crc32_checksum.dart';

typedef void Closure(bool val, Transaction response);
typedef void QRDecryptClosure(bool val, QrDecryptResponse response);

//ignore: must_be_immutable
class AirPayUAE extends StatefulWidget {
  final UserRequest user;
  final Closure? closure;
  final QRDecryptClosure? qrDecryptClosure;

  bool isShowProgress = true;


  AirPayUAE({Key? key, required this.user, this.closure,this.qrDecryptClosure}) : super(key: key);

  @override
  _AirPayUAEState createState() => new _AirPayUAEState();
}

class _AirPayUAEState extends State<AirPayUAE> {
  bool isShowProgress = true;

  // var URL = 'https://google.com.tr';
  String? url = "";
  var postdata = "";
  late final WebViewController _controller;
  num _stackToView = 1;
  String errMsg = "";
  var payURL = '';
  bool isFirst = true;
  bool isProceed = false;

  String airpayMerchantId = "";
  String airpayApikey = "";
  String airpayUsername = "";
  String airpayPassword = "";
  String transaction_session_id = "";
  String successUrl = "";
  String failedUrl = "";
  String orderId = "";

  var secret_key ='b0B1c#dsaf45ADSF5645adf0215';

  void _handleLoad(int index) {
    setState(() {
      _stackToView = index;
    });
  }

  showLoader(value) {
    setState(() {
      isShowProgress = value;
    });
  }


  Future<String> loadData() async {
    var user = widget.user;
    var actionType = widget.user.actionType;

    if (actionType != null && actionType.isNotEmpty && actionType == "IndexPay") {
      //INDEX FLOW

      Map<String, dynamic> jsonParams = {
        "merchant_id": widget.user.mercid.toString(),
        "order_id": widget.user.orderid.toString(),
        "amount": widget.user.amount.toString(),
        "discount": widget.user.discount.toString(),
        "customvar": widget.user.customvar.toString(),
        "isQR": widget.user.isQR.toString(),

      };
      String jsonString = json.encode(jsonParams);
      var merchant_response = await merchantDetailsApi(jsonString);
      if(merchant_response!= null){
        if (merchant_response.status
            .toString()
            .isNotEmpty && merchant_response.status.toString() == "200")
        {
          airpayApikey = merchant_response.data!.airpayApikey.toString();
          airpayMerchantId = merchant_response.data!.airpayMerchantId.toString();
          airpayUsername = merchant_response.data!.airpayUsername.toString();
          airpayPassword = merchant_response.data!.airpayPassword.toString();
          transaction_session_id = merchant_response.data!.transactionSessionId.toString();
          successUrl = merchant_response.data!.successUrl.toString();
          failedUrl = merchant_response.data!.failedUrl.toString();
          orderId = merchant_response.data!.orderId.toString();

          var date = new DateTime.now();
          var format = DateFormat("yyyy-MM-dd");
          var formattedDate = format.format(date);
          var temp = utf8.encode(
              '${merchant_response.data!.airpayApikey
                  .toString()}@${merchant_response.data!.airpayUsername
                  .toString()}:|:${merchant_response.data!.airpayPassword
                  .toString()}');
          var privatekey = sha256.convert(temp);


          var setAllStr =
              '${widget.user.buyerEmail}${widget.user.buyerFirstName}${widget.user
              .buyerLastName}${widget.user.buyerAddress}${widget.user.buyerCity}${widget.user
              .buyerState}${widget.user.buyerCountry}${merchant_response.data!.finalAmount
              .toString()}${merchant_response.data!.orderId
              .toString()}$formattedDate';

          // key for checksum
          var sTemp2 =
          utf8.encode('${merchant_response.data!.airpayUsername
              .toString()}~:~${merchant_response.data!.airpayPassword
              .toString()}');
          var sKey = sha256.convert(sTemp2);
          // checksum
          var sAllData = utf8.encode('$sKey@$setAllStr');

          var checksum = sha256.convert(sAllData);


          var protocolDomain = getProtoDomain(merchant_response.data!.successUrl
              .toString()); //widget.user.protoDomain
          List<int> bytes = ascii.encode(protocolDomain);
          var encoded = base64.encode(bytes);

          String productionURL = "https://payments.airpay.ae/pay/index.php"; //"https://payments.airpay.co.in/pay/index.php"; // UAE Kit Url - .ae domain
          String productionFailedURL = "https://payments.airpay.ae/error.php"; //"https://payments.airpay.co.in/error.php"; // UAE Kit Url - .ae domain

          String stagingURL = "https://uae-payments.airpay.ninja/pay/index.php"; //"https://payments.airpay.ninja/pay/index.php"; //Staging URL
          String stagingFailedURL = "https://uae-payments.airpay.ninja/error.php"; //"https://payments.airpay.ninja/error.php"; //Staging URL

          var isGateWay = (user.isStaging != null && user.isStaging == true)
              ? stagingURL
              : productionURL;
          user.failedUrl = (user.isStaging != null && user.isStaging == true)
              ? stagingFailedURL
              : productionFailedURL;

          var url = "<!DOCTYPE html>" +
              "<html>" +
              "<body onload='document.frm1.submit()'>" +
              "<form action='$isGateWay' method='post' name='frm1'>" +
              "  <input type='hidden' name='mer_dom' value='$encoded'><br>" +
              "  <input type='hidden' name='currency' value='${user
                  .currency}'><br>" +
              "  <input type='hidden' name='isocurrency' value='${user
                  .isocurrency}'><br>" +
              "  <input type='hidden' name='orderid' value='${merchant_response
                  .data!.orderId.toString()}'><br>" +
              "  <input type='hidden' name='privatekey' value='$privatekey'><br>" +
              "  <input type='hidden' name='checksum' value='$checksum'><br>" +
              "  <input type='hidden' name='mercid' value='${merchant_response
                  .data!.airpayMerchantId.toString()}'><br>" +
              "  <input type='hidden' name='buyerEmail' value='${user
                  .buyerEmail}'><br>" +
              "  <input type='hidden' name='buyerPhone' value='${user
                  .buyerPhone}'><br>" +
              "  <input type='hidden' name='buyerFirstName' value='${user
                  .buyerFirstName}'><br>" +
              "  <input type='hidden' name='buyerLastName' value='${user
                  .buyerLastName}'><br>" +
              "  <input type='hidden' name='buyerAddress' value='${user
                  .buyerAddress}'><br>" +
              "  <input type='hidden' name='buyerCity' value='${user
                  .buyerCity}'><br>" +
              "  <input type='hidden' name='buyerState' value='${user
                  .buyerState}'><br>" +
              "  <input type='hidden' name='buyerCountry' value='${user
                  .buyerCountry}'><br>" +
              "  <input type='hidden' name='buyerPinCode' value='${user
                  .buyerPinCode}'><br>" +
              "  <input type='hidden' name='amount' value='${merchant_response
                  .data!.finalAmount.toString()}'><br>" +
              "  <input type='hidden' name='chmod' value='${user.chmod}'><br>" +
              "  <input type='hidden' name='customvar' value='${user
                  .customvar}'><br>" +
              "  <input type='hidden' name='txnsubtype' value='${user
                  .txnsubtype}'><br>" +
              "  <input type='hidden' name='token' value=''><br>" +

              "</form>" +
              "</body>" +
              "</html>";
          return url;
        } else {
          //return to Merchant App - Failed Case for Emar API
          Transaction transaction = Transaction();
          transaction.TRANSACTIONSTATUS = merchant_response.status.toString();
          transaction.sTATUSMSG = merchant_response.message.toString();
          widget.closure!(false, transaction);

        }
      }else{
        //return to Merchant App

        userCancel("Service Unavailable");
      }

    }else if (actionType != null && actionType.isNotEmpty && actionType == "DirectPay") {
//DIRECT INDEX FLOW
      Map<String, dynamic> jsonParams = {
        "merchant_id": widget.user.mercid.toString(),
        "order_id": widget.user.orderid.toString(),
        "amount": widget.user.amount.toString(),
        "discount": widget.user.discount.toString(),
        "customvar": widget.user.customvar.toString(),
        "isQR": widget.user.isQR.toString(),

      };
      String jsonString = json.encode(jsonParams);
      var merchant_response =  await merchantDetailsApi(jsonString);

      if(merchant_response!= null){
        if (merchant_response.status
            .toString()
            .isNotEmpty && merchant_response.status.toString() == "200")
        {
          airpayApikey = merchant_response.data!.airpayApikey.toString();
          airpayMerchantId = merchant_response.data!.airpayMerchantId.toString();
          airpayUsername = merchant_response.data!.airpayUsername.toString();
          airpayPassword = merchant_response.data!.airpayPassword.toString();
          transaction_session_id = merchant_response.data!.transactionSessionId.toString();
          successUrl = merchant_response.data!.successUrl.toString();
          failedUrl = merchant_response.data!.failedUrl.toString();
          orderId = merchant_response.data!.orderId.toString();

          var date = new DateTime.now();
          var format = DateFormat("yyyy-MM-dd");
          var formattedDate = format.format(date);
          var temp = utf8.encode(
              '${merchant_response.data!.airpayApikey.toString()}@${merchant_response.data!.airpayUsername.toString()}:|:${merchant_response.data!.airpayPassword.toString()}');
          var privatekey = sha256.convert(temp);


          var setAllStr =
              '${widget.user.buyerEmail}${widget.user.buyerFirstName}${widget.user
              .buyerLastName}${widget.user.buyerAddress}${widget.user.buyerCity}${widget.user
              .buyerState}${widget.user.buyerCountry}${merchant_response.data!.finalAmount.toString()}${merchant_response.data!.orderId.toString()}${widget.user.channel}$formattedDate';

          // key for checksum
          var sTemp2 =
          utf8.encode('${merchant_response.data!.airpayUsername.toString()}~:~${merchant_response.data!.airpayPassword.toString()}');
          var sKey = sha256.convert(sTemp2);
          // checksum
          var sAllData = utf8.encode('$sKey@$setAllStr');


          var checksum = sha256.convert(sAllData);


          var protocolDomain = getProtoDomain(merchant_response.data!.successUrl.toString());
          List<int> bytes = ascii.encode(protocolDomain);
          var encoded = base64.encode(bytes);

          String productionURL = "https://payments.airpay.ae/pay/directindexapi.php"; //"https://payments.airpay.co.in/pay/index.php"; // UAE Kit Url - .ae domain
          String productionFailedURL = "https://payments.airpay.ae/error.php"; //"https://payments.airpay.co.in/error.php"; // UAE Kit Url - .ae domain

          String stagingURL = "https://uae-payments.airpay.ninja/pay/directindexapi.php"; //"https://payments.airpay.ninja/pay/index.php"; //Staging URL
          String stagingFailedURL = "https://uae-payments.airpay.ninja/error.php"; //"https://payments.airpay.ninja/error.php"; //Staging URL


          var isGateWay = (user.isStaging != null && user.isStaging == true)
              ? stagingURL
              : productionURL;
          user.failedUrl = (user.isStaging != null && user.isStaging == true)
              ? stagingFailedURL
              : productionFailedURL;

          var url = "<!DOCTYPE html>" +
              "<html>" +
              "<body onload='document.frm1.submit()'>" +
              "<form action='$isGateWay' method='post' name='frm1'>" +
              "  <input type='hidden' name='mer_dom' value='$encoded'><br>" +
              "  <input type='hidden' name='currency' value='${user.currency}'><br>" +
              "  <input type='hidden' name='isocurrency' value='${user.isocurrency}'><br>" +
              "  <input type='hidden' name='orderid' value='${merchant_response.data!.orderId.toString()}'><br>" +
              "  <input type='hidden' name='privatekey' value='$privatekey'><br>" +
              "  <input type='hidden' name='checksum' value='$checksum'><br>" +
              "  <input type='hidden' name='mercid' value='${merchant_response.data!.airpayMerchantId.toString()}'><br>" +
              "  <input type='hidden' name='buyerEmail' value='${user.buyerEmail}'><br>" +
              "  <input type='hidden' name='buyerPhone' value='${user.buyerPhone}'><br>" +
              "  <input type='hidden' name='buyerFirstName' value='${user.buyerFirstName}'><br>" +
              "  <input type='hidden' name='buyerLastName' value='${user.buyerLastName}'><br>" +
              "  <input type='hidden' name='buyerAddress' value='${user.buyerAddress}'><br>" +
              "  <input type='hidden' name='buyerCity' value='${user.buyerCity}'><br>" +
              "  <input type='hidden' name='buyerState' value='${user.buyerState}'><br>" +
              "  <input type='hidden' name='buyerCountry' value='${user.buyerCountry}'><br>" +
              "  <input type='hidden' name='buyerPinCode' value='${user.buyerPinCode}'><br>" +
              "  <input type='hidden' name='amount' value='${merchant_response.data!.finalAmount.toString()}'><br>" +
              "  <input type='hidden' name='chmod' value='${user.chmod}'><br>" +
              "  <input type='hidden' name='customvar' value='${user.customvar}'><br>" +
              "  <input type='hidden' name='txnsubtype' value='${user.txnsubtype}'><br>" +
              "  <input type='hidden' name='token' value='${user.token}'><br>" +
              "  <input type='hidden' name='carduniquecode' value='${user.carduniquecode}'><br>" +
              "  <input type='hidden' name='channel' value='${user.channel}'><br>" +
              "  <input type='hidden' name='card_cvv' value='${user.cardCVV}'><br>" +

              "</form>" +
              "</body>" +
              "</html>";
          return url;
        } else {
          //return to Merchant App - Failed Case for Emar API
          Transaction transaction = Transaction();
          transaction.TRANSACTIONSTATUS = merchant_response.status.toString();
          transaction.sTATUSMSG = merchant_response.message.toString();
          widget.closure!(false, transaction);
        }
      }else{
        //return to Merchant App

        userCancel("Service Unavailable");
      }
    }else{

      //New code added
      //closure(true,"Invalid Input Parameter Request");
      Transaction transaction = Transaction();
      transaction.sTATUSMSG = 'Invalid Input Parameter Request';
      widget.closure!(false, transaction);
    }
    return url?? '';
  }

  @override
  void initState()  {
    super.initState();
    checkConnection();


    WidgetsBinding.instance.addPostFrameCallback((_) {
      callAsync();
    });


  }

  callAsync() async {
    _asyncMethod();
  }



  Future<void> _asyncMethod() async {

    String errMsg = '';
    // ignore: unnecessary_null_comparison
    if (widget.user == null) {
      errMsg = 'Kindly enter your Airpay details to proceed';
    }

    if (errMsg.isNotEmpty) {
      isProceed = false;
      _showAlert(
          context,
          errMsg +
              '\n to proceed with the demo app you must enter the required details to proceed.');
      print(errMsg);
      return;
    }
    //});
    _handleLoad(1);
    this.payURL = Uri.dataFromString(await loadData(),
        mimeType: 'text/html', encoding: Encoding.getByName('utf-8'))
        .toString();

    // if (Platform.isAndroid) WebView.platform = SurfaceAndroidWebView();
    late final PlatformWebViewControllerCreationParams params;
    params = const PlatformWebViewControllerCreationParams();
    final WebViewController controller =
    WebViewController.fromPlatformCreationParams(params);
    controller
      ..setJavaScriptMode(JavaScriptMode.unrestricted)
      ..setBackgroundColor(const Color(0x00000000))
      ..setNavigationDelegate(
        NavigationDelegate(
          onProgress: (int progress) {},
          onPageStarted: (String url) {


            _handleLoad(1);

            setState(() {
              //var succesPath = getProtoDomain(widget.user.successUrl!);
              var succesPath = getProtoDomain(successUrl);
              var webURLPath = getProtoDomain(url);
              if (succesPath.contains("http://") &&
                  webURLPath.contains("https://")) {
                webURLPath = webURLPath.replaceAll("https://", "http://");
              } else if (succesPath.contains("https://") &&
                  webURLPath.contains("http://")) {
                webURLPath = webURLPath.replaceAll("http://", "https://");
              }

              if (succesPath == webURLPath) {
                // isShow = false;
                _handleLoad(1);
                fetchDetails();
              } else if (failedUrl == webURLPath) {
                userCancel('Transaction failed');
              }
            });
          },
          onPageFinished: (String url1) async {


            if (Platform.isAndroid && this.isFirst == true) {
              this.isFirst = false;
              Future.delayed(const Duration(seconds: 6), () {
                _handleLoad(0);
              });
            } else {
              _handleLoad(0);
            }

          },
          onWebResourceError: (WebResourceError error) {
            debugPrint('''
Page resource error:
  code: ${error.errorCode}
  description: ${error.description}
  errorType: ${error.errorType}
  isForMainFrame: ${error.isForMainFrame}
          ''');
          },
          onNavigationRequest: (NavigationRequest request) {
            return NavigationDecision.navigate;
          },
        ),
      )
      ..addJavaScriptChannel(
        'Toaster',
        onMessageReceived: (JavaScriptMessage message) {
          ScaffoldMessenger.of(context).showSnackBar(
            SnackBar(content: Text(message.message)),
          );
        },
      )
      ..loadRequest(Uri.parse(this.payURL));

    // #docregion platform_features
    if (controller.platform is AndroidWebViewController) {
      AndroidWebViewController.enableDebugging(true);
      (controller.platform as AndroidWebViewController)
          .setMediaPlaybackRequiresUserGesture(false);
    }
    // #enddocregion platform_features

    _controller = controller;
    isProceed = true;

    // });
  }


  checkConnection() async {
    try {
      final result = await InternetAddress.lookup('google.com');
      if (result.isNotEmpty && result[0].rawAddress.isNotEmpty) {
        print('connected');

      }

    } on SocketException catch (_) {
      _showAlert(context, 'No Internet');
      print('not connected');

    }
  }

  String getProtoDomain(String sDomain) {
    if (sDomain.isNotEmpty == true) {
      int slashslash = sDomain.indexOf("//") + 2;
      return sDomain.substring(0, sDomain.indexOf("/", slashslash));
    }
    return '';
  }

//a.php api response
  fetchDetails() async {
    var user = widget.user;

    String stagingDataURL = "https://uae-payments.airpay.ninja/sdk/a.php";
    String productionDataURL = "https://payments.airpay.ae/sdk/a.php";
    var isGateWay = (user.isStaging != null && user.isStaging == true)
        ? stagingDataURL
        : productionDataURL;

    String urlString = isGateWay;
    var date = new DateTime.now();
    var format = DateFormat("dd/MM/yyyy HH:mm:ss");
    var formattedDate = format.format(date);
    format = DateFormat("yyyy HH:mm:ss");
    var formattedDate2 = format.format(date);

    var temp = utf8.encode(
        '${airpayApikey}@${airpayUsername}:|:${airpayPassword}');
    var privatekey = sha256.convert(temp);

    var setAllData = utf8.encode(
        '$privatekey${orderId}${airpayMerchantId}$formattedDate2');
    var checksum = md5.convert(setAllData);

    //For Encyption -

    Map<String, dynamic> jsonParams = {
      "privatekey": privatekey.toString(),
      "orderid":  orderId,
      "mercid": airpayMerchantId,
      "checksum": checksum.toString(),
      "datetime": formattedDate,
    };
    String jsonString = json.encode(jsonParams);
    //var encrypt_request = encrypt256Str(jsonString,widget.user.secret.toString());
    var encrypt_request = encrypt256Str(jsonString,airpayApikey);

    FormData formData = new FormData.fromMap({
      'enc': encrypt_request,
      'mercid': widget.user.mercid

    });

    Dio dio = new Dio();
    dio.options.headers['Accept'] =
    'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8';


    var response = await dio.post(urlString, data: formData);
    final myTransformer = Xml2Json();
    var stingDAta = response.data.toString();


    // Parse the response body as JSON
    Map<String, dynamic> parsedResponse = jsonDecode(response.data.toString());
    // Access the 'query' property in the parsed response
    String enc = parsedResponse['enc'];
    stingDAta = decrypt256Str(enc,airpayApikey);

    stingDAta = stingDAta.replaceAll("<![CDATA[", "").replaceAll("]]>", "");
    myTransformer.parse(stingDAta);
    var document = myTransformer.toParker();
    var data = json.decode(document);
    Transaction transaction = Transaction();
    if (data != null && data['RESPONSE'] != null) {
      if (data['RESPONSE']['TRANSACTION'] != null) {
        //transaction.sTATUS = '500';
        transaction = Transaction.fromJson(data['RESPONSE']['TRANSACTION']);
      }else{
        transaction.sTATUSMSG = 'No response from server';

      }
    }else{
      transaction.sTATUSMSG = 'No response from server';
    }

    //Calling Acknowled API Implementation -


    Map<String, dynamic> jsonResponseParams = {
      "order_id":  transaction.mERCHANTTRANSACTIONID,
      "merchant_id": airpayMerchantId,
      "airpay_id": transaction.tRANSACTIONID,
      "txn_mode": transaction.cHMOD,
      "final_amount": transaction.bILLEDAMOUNT,
      "status": transaction.sTATUSMSG,
      "transaction_session_id": transaction_session_id,
      "statuscode": transaction.sTATUS,
    };
    String jsonResponseString = json.encode(jsonResponseParams);

    AckResponseApi(jsonResponseString, transaction);

    //SecureHash Logic START -
    var txtStsMsg = transaction.sTATUSMSG;
    var txtSts = transaction.tRANSACTIONSTATUS;
    if (txtStsMsg == '') {
      txtStsMsg = transaction.sTATUSMSG;
      txtSts = transaction.tRANSACTIONSTATUS;
    }
    if (txtStsMsg == 'Invalid Checksum') {
      // txtStsMsg = "Transaction Canceled";
    }

    var transid = transaction.mERCHANTTRANSACTIONID;
    var apTransactionID = transaction.tRANSACTIONID;
    var amount = transaction.tRANSACTIONAMT;
    var transtatus = transaction.tRANSACTIONSTATUS;
    var message = transaction.sTATUSMSG;
    String? customer_vpa ="";
    var customer_fvpa ="";
    var chmode = transaction.cHMOD;
    var secureHash = transaction.aPSECUREHASH;


    //(!TextUtils.isEmpty(transaction.getChMode()) && transaction.getChMode().equalsIgnoreCase("upi")){
    if(!chmode.toString().isEmpty && chmode.toString() == "upi" ){
      customer_vpa = transaction.cUSTOMERVPA;
      customer_fvpa = ":$customer_vpa";
    }
    var merchantid = airpayMerchantId; //Please enter Merchant Id
    var username = airpayUsername;   //Please enter Username


// Calculate the CRC32 checksum

    var sParam =
        '${transid}:${apTransactionID}:${amount}:${transtatus}:${message}:${merchantid}:${username}$customer_fvpa';

    var checkSumResult = Crc32.calculate(sParam);
    //SecureHash Logic END -
    if (transaction.tRANSACTIONSTATUS != null) {
      if (checkSumResult.toString() == secureHash.toString()) {
        widget.closure!(true, transaction);
      } else{
        transaction.sTATUSMSG = '${transaction.sTATUSMSG} - Secure Hash mis-matched';
        widget.closure!(false, transaction);
      }
    }else{
      widget.closure!(false, transaction);
    }
  }

  userCancel(String withMessage) {
    var trans = Transaction();
    //trans.sTATUS = '500';
    trans.tRANSACTIONREASON = withMessage;
    widget.closure!(false, trans);
  }



  // Additional Code changes for QR Decrypt API Implementation - Start


  // Function to make API call
  Future<void> AckResponseApi(ack_request, Transaction transaction) async {
    //final url = 'https://emar.airpay.ninja/node/api/txnResponse';


    String stagingDataURL = 'https://emar.airpay.ninja/node/api/txnResponse';
    String productionDataURL = 'https://api.airpay.ae/node/api/txnResponse';
    String stagingAirpayKey = "AL0013AL1688469524";
    String stagingPartnerKey = "AF0064AF1688469603";
    String productionAirpayKey = "AL0015AL1693490184";
    String productionPartnerKey = "AF0038AF1693490253";

    var isGateWay = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingDataURL
        : productionDataURL;

    String url = isGateWay;

    var airpayKey = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingAirpayKey
        : productionAirpayKey;
    String airpay_Key = airpayKey;

    var partnerKey = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingPartnerKey
        : productionPartnerKey;
    String partner_Key = partnerKey;


    var acknowledgement_request = encryptStr(ack_request);
    var checksum_header = sha256.convert(utf8.encode('${acknowledgement_request.toString()}${secret_key.toString()}'));

    // Define your request headers
    Map<String, String> headers = {
      'AIRPAYKEY': airpay_Key,
      'PARTNERKEY': partner_Key,
      'CHECKSUM': checksum_header.toString(),
      'Content-Type': 'application/x-www-form-urlencoded'

    };

    // Define your request body
    Map<String, dynamic> requestBody = {
      'query': acknowledgement_request,
    };


    // Send POST request to the API
    final response = await http.post(
      Uri.parse(url),
      headers: headers,
      body: requestBody,

    );

    if (response.statusCode == 200) {
      // Request successful, handle the response here

      // Parse the response body as JSON
      Map<String, dynamic> parsedResponse_status = jsonDecode(response.body);

      // Check if the 'status' key is present in the response
      if (parsedResponse_status.containsKey('status')) {
        // Access the 'status' and 'message' properties in the parsed response
        int status = parsedResponse_status['status'];
        String message = parsedResponse_status['message'];


        return;
        // The rest of your code...
      } else  if (parsedResponse_status.containsKey('query')) {
        // Parse the response body as JSON
        Map<String, dynamic> parsedResponse = jsonDecode(response.body);
        // Access the 'query' property in the parsed response
        String query = parsedResponse['query'];
        String decrypt_response = decryptStr(query);

        return;
      }

    } else {
      // Request failed, handle the error here
      print('API call failed');

    }
  }



  // Function to make API call
  Future<MerchantDetailResponse?> merchantDetailsApi(json_request) async {


    String stagingDataURL = 'https://emar.airpay.ninja/node/api/getMerchantDetails';
    String productionDataURL = 'https://api.airpay.ae/node/api/getMerchantDetails';
    String stagingAirpayKey = "AL0013AL1688469524";
    String stagingPartnerKey = "AF0064AF1688469603";
    String productionAirpayKey = "AL0015AL1693490184";
    String productionPartnerKey = "AF0038AF1693490253";

    var isGateWay = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingDataURL
        : productionDataURL;

    String url = isGateWay;


    var airpayKey = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingAirpayKey
        : productionAirpayKey;
    String airpay_Key = airpayKey;

    var partnerKey = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingPartnerKey
        : productionPartnerKey;
    String partner_Key = partnerKey;


    var merchant_request = encryptStr(json_request);
    var checksum_header = sha256.convert(utf8.encode('${merchant_request.toString()}${secret_key.toString()}'));
    // Define your request headers
    Map<String, String> headers = {
      'AIRPAYKEY': airpay_Key,
      'PARTNERKEY': partner_Key,
      'CHECKSUM': checksum_header.toString(),
      'Content-Type': 'application/x-www-form-urlencoded'

    };

    // Define your request body
    Map<String, dynamic> requestBody = {
      'query': merchant_request,
    };


    // Send POST request to the API
    final response = await http.post(
      Uri.parse(url),
      headers: headers,
      body: requestBody,

    );

    if (response.statusCode == 200) {
      // Request successful, handle the response here

      // Parse the response body as JSON
      Map<String, dynamic> parsedResponse_status = jsonDecode(response.body);

      // Check if the 'status' key is present in the response
      if (parsedResponse_status.containsKey('status')) {
        // Access the 'status' and 'message' properties in the parsed response
        int status = parsedResponse_status['status'];
        String message = parsedResponse_status['message'];

        return MerchantDetailResponse(status: status,message: message,data: null);
        // The rest of your code...
      } else  if (parsedResponse_status.containsKey('query')) {
        // Parse the response body as JSON
        Map<String, dynamic> parsedResponse = jsonDecode(response.body);
        // Access the 'query' property in the parsed response
        String query = parsedResponse['query'];
        String decrypt_response = decryptStr(query);
        var merchant_details_response = merchantDetailResponseFromJson(
            decrypt_response);

        return merchant_details_response;
      }
      else{
        return null;
      }
    } else {
      // Request failed, handle the error here
      print('API call failed');
      return null;
    }
  }

//Decryption code - MS
  String decryptStr(String fullResponse){
    fullResponse.split("|");

    String ivFromResponse = fullResponse.split("|")[0];
    String encryptedResponse = fullResponse.split("|")[1];

    final key = encrypt.Key.fromUtf8("5PbpCqeioHQ3MWj6dJVeluXD");
    final iv = encrypt.IV.fromUtf8(ivFromResponse);

    final encrypter = encrypt.Encrypter(encrypt.AES(key, mode: encrypt.AESMode.cbc)); //, padding: 'PKCS7'


    final encrypted = encrypt.Encrypted.fromBase16(encryptedResponse);
    final decrypted = encrypter.decrypt(encrypted, iv: iv);

    return decrypted;
  }

  //Encryption code - MS
  String encryptStr(String plainText){

    var generatedIV = generate16DigitNumber();
    final key = encrypt.Key.fromUtf8("5PbpCqeioHQ3MWj6dJVeluXD");
    // final iv = encrypt.IV.fromSecureRandom(16);
    final iv = encrypt.IV.fromUtf8(generatedIV);
    final encrypter = encrypt.Encrypter(encrypt.AES(key,mode:encrypt.AESMode.cbc));

    final encrypted = encrypter.encrypt(plainText, iv: iv);
    final decrypted = encrypter.decrypt(encrypted, iv: iv);

    return generatedIV+"|"+encrypted.base16;
  }


  String encrypt256_Str(String plainText, String secretKey){

    // final key = encrypt.Key.fromUtf8("5PbpCqeioHQ3MWj6dJVeluXD");
    var secretkey_md5 = md5.convert(utf8.encode(secretKey));
    final key = encrypt.Key.fromUtf8(secretkey_md5.toString()); //32
    final iv = encrypt.IV.fromSecureRandom(16);
    final encrypter = encrypt.Encrypter(encrypt.AES(key,mode:encrypt.AESMode.cbc));

    final encrypted = encrypter.encrypt(plainText, iv: iv);
    final decrypted = encrypter.decrypt(encrypted, iv: iv);

    // return "$iv|$encrypted.base64";
    var str = base64Encode(utf8.encode(encrypted.base64));
    return "$iv+$str";
  }

  String generate16DigitNumber() {
    Random random = Random();
    String number = '';
    for (int i = 0; i < 16; i++) {
      number += random.nextInt(10).toString();
    }
    return number;
  }

  //Encryption code - Acq
  String encrypt256Str(String plainText, String secretKey){

    var generatedIV = generate16DigitNumber();

    var secretkey_md5 = md5.convert(utf8.encode(secretKey));

    final key = encrypt.Key.fromUtf8(secretkey_md5.toString()); //32
    final iv = encrypt.IV.fromUtf8(generatedIV);
    final encrypter = encrypt.Encrypter(encrypt.AES(key,mode:encrypt.AESMode.cbc));

    final encrypted = encrypter.encrypt(plainText, iv: iv);
    //  final decrypted = encrypter.decrypt(encrypted, iv: iv);

    return generatedIV + encrypted.base64;
  }

  //Decryption code - Acq
  String decrypt256Str(String fullResponse, String secretKey){
    
    String ivFromResponse = fullResponse.substring(0, 16);
    String encryptedResponse = fullResponse.substring(16);


    var secretkey_md5 = md5.convert(utf8.encode(secretKey));

    final key = encrypt.Key.fromUtf8(secretkey_md5.toString());
    final iv = encrypt.IV.fromUtf8(ivFromResponse);

    final encrypter = encrypt.Encrypter(encrypt.AES(key, mode: encrypt.AESMode.cbc)); //, padding: 'PKCS7'


    final encrypted = encrypt.Encrypted.fromBase64(encryptedResponse);
    final decrypted = encrypter.decrypt(encrypted, iv: iv);

    return decrypted;
  }



// Encryption code - END

//Additional Code changes for QR Decrypt API Implementation - End


  _showAlert(context, message) async {
    this.errMsg = message;
    await showDialog<String>(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        String title = "Airpay";
        String message1 = message;
        return Platform.isIOS
            ? new CupertinoAlertDialog(
          title: Text(title),
          content: Text(message1),
          actions: <Widget>[
            new Container(
              margin: EdgeInsets.all(8.0),
              child: ElevatedButton(
                // padding: EdgeInsets.all(12.0),
                onPressed: () {
                  Navigator.pop(context);
                },
                // color: Colors.blue[900],
                child: Text(
                  'Okay',
                  style: TextStyle(color: Colors.white, fontSize: 24.0),
                ),
              ),
            )
          ],
        )
            : new AlertDialog(
          title: Text(title),
          content: Container(
              height: 140.0,
              width: 400.0,
              child: ListView.builder(
                  shrinkWrap: true,
                  itemCount: 1,
                  itemBuilder: (BuildContext context, int index) {
                    return new Column(children: <Widget>[
                      Text(message1),
                    ]);
                  })),
          actions: <Widget>[
            new Container(
              margin: EdgeInsets.all(8.0),
              child: ElevatedButton(
                onPressed: () {
                  Navigator.pop(context);
                },
                child: Text(
                  'Okay',
                  style: TextStyle(color: Colors.white, fontSize: 24.0),
                ),
              ),
            )
          ],
        );
      },
    );
  }

  _showConfirmation(context, message) async {
    await showDialog<String>(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        String title = "Airpay";
        String message1 = message;
        return Platform.isIOS
            ? new CupertinoAlertDialog(
          title: Text(title),
          content: Text(message1),
          actions: <Widget>[
            new Container(
              margin: EdgeInsets.all(0),
              child: ElevatedButton(
                // padding: EdgeInsets.all(8.0),
                onPressed: () {
                  Navigator.pop(context);
                  userCancel('User canceled');
                },
                // color: Colors.blue[900],
                child: Text(
                  'Yes',
                  style: TextStyle(color: Colors.white, fontSize: 24.0),
                ),
              ),
            ),
            new Container(
              margin: EdgeInsets.all(0),
              child: ElevatedButton(
                // padding: EdgeInsets.all(8.0),
                onPressed: () {
                  Navigator.pop(context);
                },
                // color: Colors.blue[900],
                child: Text(
                  'No',
                  style: TextStyle(color: Colors.white, fontSize: 24.0),
                ),
              ),
            )
          ],
        )
            : new AlertDialog(
          title: Text(title),
          content: Text(message1),
          actions: <Widget>[
            new Container(
              margin: EdgeInsets.all(0),
              child: ElevatedButton(
                // padding: EdgeInsets.all(2.0),
                onPressed: () {
                  Navigator.pop(context);
                  userCancel('User canceled');
                },
                // color: Colors.blue[900],
                child: Text(
                  'Yes',
                  style: TextStyle(color: Colors.white, fontSize: 24.0),
                ),
              ),
            ),
            new Container(
              margin: EdgeInsets.all(0),
              child: ElevatedButton(
                // padding: EdgeInsets.all(2.0),
                onPressed: () {
                  Navigator.pop(context);
                },
                // color: Colors.blue[900],
                child: Text(
                  'No',
                  style: TextStyle(color: Colors.white, fontSize: 24.0),
                ),
              ),
            ),
          ],
        );
      },
    );
  }

  // InAppWebViewController _webViewController;
  @override
  Widget build(BuildContext context) {
    return SafeArea(
        child: Scaffold(
            appBar: AppBar(
              centerTitle: true,
              title: Image.asset(
                'assets/airpayLogo.png',
                height: 40,
                color: Colors.white,
                width: 200,
              ),
              leading: IconButton(
                icon: Icon(Icons.close),
                onPressed: () => {
                  _showConfirmation(
                      context, "Did you want to cancel this transaction ?")
                },
              ),
              backgroundColor: Colors.blue[900],
              actions: <Widget>[],
            ),
            body: (isProceed
                ? IndexedStack(index: _stackToView as int?, children: [
              Column(children: <Widget>[
                Expanded(child: WebViewWidget(controller: _controller)),
              ]),
              Container(
                  child: Center(
                      child: SpinKitCircle(
                        color: Colors.blue[900],
                        size: 50.0,
                      )))
            ])
                : Container(child: Center(child: Text(this.errMsg))))));
  }

}

